"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasApiKeySettings = void 0;
const axios_1 = require("axios");
const openid_client_1 = require("openid-client");
const node_fetch_1 = require("node-fetch");
async function auth(settings) {
    return hasApiKeySettings(settings) ? apiKeyClient(settings) : openIdClient(settings);
}
exports.default = auth;
async function apiKeyClient({ apiKey }) {
    let cookie = '';
    const authHeader = 'X-External-Authentication';
    async function requestUserData(nexusUrl) {
        const userUrl = `${nexusUrl}/user`;
        const requestSettings = {
            method: 'GET',
            headers: { [authHeader]: apiKey },
        };
        const response = (await axios_1.default(userUrl, requestSettings));
        if (response.data) {
            const userData = response.data;
            if (response.headers) {
                cookie = getCookieValue(response.headers['set-cookie']);
            }
            return userData;
        }
        return null;
    }
    /**
     * Initializes the authentication for the client
     *
     * @param system A reference to the bf-lib system api
     */
    async function connect(nexusUrl) {
        try {
            return requestUserData(nexusUrl);
        }
        catch (e) {
            return null;
        }
    }
    function afterConnect(system) {
        system.setHttpHeader('Cookie', cookie);
        system.setHttpHeader(authHeader, apiKey);
    }
    /**
     * Will attempt to refresh the authentication data for the client
     *
     * First, the cached access token is used. If that fails, a new token set is requested.
     * One more attempt is then made with the new token set. If that fails, the reconnect
     * operation returns a value of null.
     *
     * @param system A reference to the bf-lib system api
     */
    function reconnect(system) {
        return requestUserData(system.nexus.getUrl());
    }
    /**
     * Revokes the client's session and then ends the running process
     *
     * @param system A reference to the bf-lib system api
     */
    async function disconnect(system) {
        try {
            await node_fetch_1.default(`${system.nexus.getUrl()}/logout`);
        }
        catch { }
        process.exit(0);
    }
    async function getAccessToken() {
        return apiKey;
    }
    const clientAuth = {
        connect,
        afterConnect,
        reconnect,
        disconnect,
        getAccessToken,
    };
    return clientAuth;
}
async function openIdClient({ clientId, clientSecret, issuerURI, scopes, }) {
    scopes = scopes || [];
    openid_client_1.custom.setHttpOptionsDefaults({
        timeout: 60000,
    });
    const authHeader = 'X-External-Client';
    const issuer = await openid_client_1.Issuer.discover(issuerURI);
    const client = new issuer.Client({
        client_id: clientId,
        client_secret: clientSecret,
        scope: `openid profile email ${scopes.join(' ')}`,
    });
    let cookie = '';
    let tokenSet;
    async function getAccessToken() {
        try {
            if (!tokenSet || !tokenSet.refresh_token) {
                tokenSet = await client.grant({
                    grant_type: 'client_credentials',
                });
            }
            else if (tokenSet.expired()) {
                tokenSet = await client.refresh(tokenSet);
            }
            if (!tokenSet.access_token) {
                console.error('no access token could be generated');
                return null;
            }
            return tokenSet.access_token;
        }
        catch (err) {
            if (err.message.includes('Refresh token expired')) {
                tokenSet = null;
                return getAccessToken();
            }
            else {
                console.error('an error occurred while trying generate an access token:', err.message);
                return null;
            }
        }
    }
    async function requestUserData(nexusUrl, tryRefresh = false) {
        var _a;
        const userUrl = `${nexusUrl}/user`;
        const requestSettings = {
            method: 'GET',
            headers: { [authHeader]: (_a = tokenSet === null || tokenSet === void 0 ? void 0 : tokenSet.access_token) !== null && _a !== void 0 ? _a : '' },
        };
        const response = (await axios_1.default(userUrl, requestSettings));
        if (response.data) {
            const userData = response.data;
            if (response.headers) {
                cookie = getCookieValue(response.headers['set-cookie']);
            }
            return userData;
        }
        if (tryRefresh && tokenSet) {
            if (!isValidString(tokenSet.refresh_token)) {
                return null;
            }
            const newTokens = await client.refresh(tokenSet);
            if (!newTokens || !isValidString(newTokens.access_token)) {
                return null;
            }
            tokenSet = newTokens;
            return requestUserData(nexusUrl);
        }
        return null;
    }
    /**
     * Initializes the authentication for the client
     *
     * @param system A reference to the bf-lib system api
     */
    async function connect(nexusUrl) {
        try {
            const tokens = await client.grant({
                grant_type: 'client_credentials',
            });
            if (!tokens || !isValidString(tokens.access_token)) {
                return null;
            }
            tokenSet = tokens;
            return requestUserData(nexusUrl);
        }
        catch (e) {
            return null;
        }
    }
    function afterConnect(system) {
        system.setHttpHeader('Cookie', cookie);
    }
    /**
     * Will attempt to refresh the authentication data for the client
     *
     * First, the cached access token is used. If that fails, a new token set is requested.
     * One more attempt is then made with the new token set. If that fails, the reconnect
     * operation returns a value of null.
     *
     * @param system A reference to the bf-lib system api
     */
    function reconnect(system) {
        return requestUserData(system.nexus.getUrl(), true);
    }
    /**
     * Revokes the client's session and then ends the running process
     *
     * @param system A reference to the bf-lib system api
     */
    async function disconnect(system) {
        try {
            await node_fetch_1.default(`${system.nexus.getUrl()}/logout`);
        }
        catch { }
        process.exit(0);
    }
    const clientAuth = {
        connect,
        afterConnect,
        reconnect,
        disconnect,
        getAccessToken,
    };
    return clientAuth;
}
function isValidString(value) {
    return typeof value === 'string' && value.length > 0;
}
function getCookieValue(value) {
    if (!value) {
        return '';
    }
    value = Array.isArray(value) ? value[0] : value;
    return value.split(';')[0];
}
function hasApiKeySettings(settings) {
    return settings.hasOwnProperty('apiKey');
}
exports.hasApiKeySettings = hasApiKeySettings;
