"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encryptionKeyHash = exports.removePemFormatting = exports.formatRsaKeyString = exports.getEnvironmentSecretPublicKey = exports.getEnvironmentSecretPrivateKey = exports.rsaDecryptData = exports.rsaEncryptData = exports.decodeSecretValue = void 0;
const common_1 = require("bf-lib/common");
const crypto = require("crypto");
const Encryption_Consts_1 = require("./Encryption.Consts");
const Encryption_Types_1 = require("./Encryption.Types");
/**
 * Decode a secret value
 *
 * @param encodedSecretValue A secret value which has been encoded
 * @param privateKey A PEM formatted RSA private key
 */
function decodeSecretValue(encodedSecretValue, privateKey) {
    const secretValueSections = encodedSecretValue.split(':');
    const format = secretValueSections[0] || '';
    switch (format) {
        case Encryption_Types_1.SecretValueEncodingType.RAW: {
            const encodedData = secretValueSections[1] || '';
            return Buffer.from(encodedData, 'base64').toString();
        }
        case Encryption_Types_1.SecretValueEncodingType.RSA: {
            if (!privateKey) {
                throw new Error('rsa encoded secret values require a private key to decrypt');
            }
            const encryptedData = secretValueSections[2] || '';
            return rsaDecryptData(privateKey, encryptedData);
        }
        default: {
            throw new Error(`invalid secret value type: ${format}`);
        }
    }
}
exports.decodeSecretValue = decodeSecretValue;
/**
 * Encrypt data using an RSA public key
 *
 * @param publicKey A PEM formatted RSA public key
 * @param unencryptedData Data that will be encrypted
 */
function rsaEncryptData(publicKey, unencryptedData) {
    const encrypted = crypto.publicEncrypt(publicKey, Buffer.from(unencryptedData));
    return encrypted.toString('base64');
}
exports.rsaEncryptData = rsaEncryptData;
/**
 * Decrypt data using an RSA private key
 *
 * @param privateKey A PEM formatted RSA private key
 * @param encryptedData Data that will be decrypted
 */
function rsaDecryptData(privateKey, encryptedData) {
    const decrypted = crypto.privateDecrypt(privateKey, Buffer.from(encryptedData, 'base64'));
    return decrypted.toString('utf8');
}
exports.rsaDecryptData = rsaDecryptData;
/**
 * Get an RSA private key used to decrypt secret values associated with a particular block-5 module
 *
 * @param moduleName The name of a block-5 module
 */
function getEnvironmentSecretPrivateKey(moduleName) {
    const envKey = `${Encryption_Consts_1.ENV_SECRET_PRIVATE_KEY_PREFIX}${common_1.toUpperSnakeCase(moduleName)}`;
    if (process.env.hasOwnProperty(envKey)) {
        return formatRsaKeyString(process.env[envKey] || '', Encryption_Types_1.AsymmetricKeyType.PRIVATE);
    }
    throw new Error(`could not find a secrets private key for the following module: ${moduleName}`);
}
exports.getEnvironmentSecretPrivateKey = getEnvironmentSecretPrivateKey;
/**
 * Get an RSA public key used to encrypt secret values associated with a particular block-5 module
 *
 * @param moduleName The name of a block-5 module
 */
function getEnvironmentSecretPublicKey(moduleName) {
    const envKey = `${Encryption_Consts_1.ENV_SECRET_PUBLIC_KEY_PREFIX}${common_1.toUpperSnakeCase(moduleName)}`;
    if (process.env.hasOwnProperty(envKey)) {
        return formatRsaKeyString(process.env[envKey] || '', Encryption_Types_1.AsymmetricKeyType.PUBLIC);
    }
    throw new Error(`could not find a secrets public key for the following module: ${moduleName}`);
}
exports.getEnvironmentSecretPublicKey = getEnvironmentSecretPublicKey;
/**
 * Convert an RSA key string into PEM format
 *
 * @param keyString An RSA key in the form of a single line string
 * @param keyType The RSA key type
 */
function formatRsaKeyString(keyString, keyType) {
    const actualKeyType = getKeyType(keyString, keyType);
    const formattedKey = chunkSubstr(removePemFormatting(keyString), 64).join('\n');
    switch (actualKeyType) {
        case Encryption_Types_1.AsymmetricKeyType.PUBLIC:
            return addPublicKeyHeaders(formattedKey);
        case Encryption_Types_1.AsymmetricKeyType.PRIVATE:
            return addPrivateKeyHeaders(formattedKey);
    }
}
exports.formatRsaKeyString = formatRsaKeyString;
/**
 * Remove any characters that would be in a PEM formatted RSA key
 *
 * @param keyString An RSA key string
 */
function removePemFormatting(keyString) {
    return keyString
        .replace(/-*[^-]*KEY-*/g, '')
        .replace(/\n/g, '')
        .replace(/\\n/g, '')
        .replace(/ /g, '');
}
exports.removePemFormatting = removePemFormatting;
/**
 * Create a hash for an encryption key
 *
 * @param keyString An RSA key string
 */
function encryptionKeyHash(keyString) {
    return crypto
        .createHash('sha256')
        .update(removePemFormatting(keyString), 'utf8')
        .digest()
        .toString('base64')
        .substr(0, 8);
}
exports.encryptionKeyHash = encryptionKeyHash;
/* ~~~ Private Utils ~~~ */
function getKeyType(keyString, defaultTo) {
    if (Encryption_Consts_1.PUBLIC_KEY_HEADER_PATTERN.test(keyString)) {
        return Encryption_Types_1.AsymmetricKeyType.PUBLIC;
    }
    if (Encryption_Consts_1.PRIVATE_KEY_HEADER_PATTERN.test(keyString)) {
        return Encryption_Types_1.AsymmetricKeyType.PRIVATE;
    }
    return defaultTo;
}
function addPrivateKeyHeaders(keyString) {
    return `-----BEGIN RSA PRIVATE KEY-----
${keyString}
-----END RSA PRIVATE KEY-----`;
}
function addPublicKeyHeaders(keyString) {
    return `-----BEGIN PUBLIC KEY-----
${keyString}
-----END PUBLIC KEY-----`;
}
function chunkSubstr(str, size) {
    const numChunks = Math.ceil(str.length / size);
    const chunks = new Array(numChunks);
    for (let i = 0, o = 0; i < numChunks; ++i, o += size) {
        chunks[i] = str.substr(o, size);
    }
    return chunks;
}
